@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -39,7 +39,7 @@ class DynamicManager(providers.ProviderHandler):
     """
         handles matching of songs for dynamic playlists
     """
-    def __init__(self, collection):
+    def __init__(self, collection=[]):
         providers.ProviderHandler.__init__(self, "dynamic_playlists")
         self.buffersize = settings.get_option("playback/dynamic_buffer", 5)
         self.collection = collection
@@ -67,10 +67,11 @@ class DynamicManager(providers.ProviderHandler):
         random.shuffle(artists)
         i = 0
         while (limit > len(tracks) or limit == -1) and i < len(artists):
-            artist = artists[i][1]
+            artist = artists[i][1].replace('"', '\\\"')
             i += 1
-            choices = [x.track for x in search.search_tracks_from_string(
-                self.collection, 'artist=="%s"'%artist.lower().replace('"', ''))]
+            searchres = search.search_tracks_from_string(
+                self.collection, 'artist=="%s"'%artist, case_sensitive=False)
+            choices = [x.track for x in searchres]
             if choices == []:
                 continue
             random.shuffle(choices)
@@ -143,27 +144,35 @@ class DynamicManager(providers.ProviderHandler):
             adds tracks to playlists as needed.
             called when the position of a playlist changes.
         """
-        if not playlist: # or not playlist.is_dynamic():
+        if not playlist:
             return
-        current_pos = playlist.get_current_pos()
+        current_pos = playlist.current_position
         if current_pos < 0 or current_pos >= len(playlist):
             return
         needed = self.buffersize - (len(playlist) - current_pos)
 
         if needed < 1:
             needed = 1
-        curr = playlist.get_current()
-        tracks = self.find_similar_tracks(curr, needed,
-                playlist.get_tracks())
+        curr = playlist.current
 
-        time.sleep(5)   # wait five seconds before adding to allow for skips
-                        # we're searching for new tracks during this time
-                        # anyway so its not wasted
-        if playlist.get_current_pos() != current_pos:
+        starttime = time.time()
+        tracks = self.find_similar_tracks(curr, needed,
+                playlist)
+
+        remainingtime = 5 - (time.time()-starttime)
+
+        if remainingtime > 0:
+            time.sleep(remainingtime)
+
+        if playlist.current_position != current_pos:
             return # we skipped in that 5 seconds, so ignore it
-        playlist.add_tracks(tracks)
+        playlist.extend(tracks)
         logger.debug("Added %s tracks." % len(tracks))
 
+
+MANAGER = DynamicManager()
+
+
 class DynamicSource(object):
     def __init__(self):
         pass
