@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -25,14 +25,14 @@
 # from your version.
 
 
-
 import os
+import sys
 import gio
 
 from xl.metadata._base import BaseFormat, NotWritable, NotReadable
 import urlparse
 
-from xl.metadata import (ape, asf, flac, mod, mp3, mp4, mpc, ogg, sid, speex,
+from xl.metadata import (ape, asf, flac, mka, mod, mp3, mp4, mpc, ogg, sid, speex,
         tta, wav, wv)
 
 #: dictionary mapping extensions to Format classes.
@@ -43,6 +43,7 @@ formats = {
         'aiff'  : wav.WavFormat,
         'ape'   : ape.MonkeysFormat,
         'amf'   : mod.ModFormat,
+        'asf'   : asf.AsfFormat,
         'au'    : wav.WavFormat,
         'dsm'   : mod.ModFormat,
         'far'   : mod.ModFormat,
@@ -50,7 +51,7 @@ formats = {
         'it'    : mod.ModFormat,
         'm4a'   : mp4.MP4Format,
         'med'   : mod.ModFormat,
-        'mka'   : None,
+        'mka'   : mka.MkaFormat,
         'mp2'   : mp3.MP3Format,
         'mp3'   : mp3.MP3Format,
         'mp4'   : mp4.MP4Format,
@@ -61,6 +62,7 @@ formats = {
         'mtm'   : mod.ModFormat,
         'oga'   : ogg.OggFormat,
         'ogg'   : ogg.OggFormat,
+        'ogx'   : ogg.OggFormat,
         'okt'   : mod.ModFormat,
         'ra'    : None,
         'ram'   : None,
@@ -91,6 +93,19 @@ def get_format(loc):
     loc = gio.File(loc).get_path()
     if not loc:
         return None
+        
+    # XXX: The path that we get from GIO is, for some reason, in UTF-8.
+    # Bug? Intended? No idea.
+    
+    # Oddly enough, if you have a non-utf8 compatible filename (such as
+    # a file from windows), then it will just return that string without 
+    # converting it (but in a form that os.path will handle). Go figure.
+    
+    try:
+        loc = loc.decode('utf-8')
+    except UnicodeDecodeError:
+        pass
+            
     ext = os.path.splitext(loc)[1]
     ext = ext[1:] # remove the pesky .
     ext = ext.lower()
