@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -24,11 +24,15 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
+from collections import namedtuple
 import copy
 import gio
 
 INFO_TAGS = ['__bitrate', '__length']
 
+# Generic description of cover images
+CoverImage = namedtuple('CoverImage', 'type desc mime data')
+
 class NotWritable(Exception):
     pass
 
@@ -52,20 +56,22 @@ class BaseFormat(object):
     # to get every single cover tag name and would probably suit our needs
     # better. perhaps any field with \n (lyrics) or >4KB (covers) would
     # work for a condition.
-    ignore_tags = ['coverart', 'cover', 'lyrics', 'Cover Art (front)']
+    ignore_tags = ['metadata_block_picture', 'coverart', 'cover', 'lyrics', 'Cover Art (front)']
 
     def __init__(self, loc):
         """
+            Raises :class:`NotReadable` if the file cannot be
+            opened for some reason.
+
             :param loc: absolute path to the file to read
                 (note - this may change to accept gio uris in the future)
-            raises NotReadable if the file cannot be opened for some reason.
         """
         self.loc = loc
         self.open = False
         self.mutagen = None
-        self.load()
         self._reverse_mapping = dict((
             (v,k) for k,v in self.tag_mapping.iteritems() ))
+        self.load()
 
     def load(self):
         """
@@ -119,7 +125,7 @@ class BaseFormat(object):
             # __ is used to denote exaile's internal tags, so we skip
             # loading them to avoid conflicts. usually this shouldn't be
             # an issue.
-            if t.startswith("__"):
+            if isinstance(t, basestring) and t.startswith("__"):
                 continue
             tags.append(t)
         alltags = self.read_tags(tags)
@@ -150,6 +156,8 @@ class BaseFormat(object):
                     t = self._get_tag(raw, self.tag_mapping[tag])
                     if type(t) in [str, unicode]:
                         t = [t]
+                    elif isinstance(t, list):
+                        pass
                     else:
                         try:
                             t = [unicode(u) for u in list(t)]
@@ -189,14 +197,17 @@ class BaseFormat(object):
         else:
             tagdict = copy.deepcopy(tagdict)
             raw = self._get_raw()
-            # add tags if it doesn't have them
-            try:
-                raw.add_tags()
-            except:
-#            except (ValueError, NotImplementedError):
-                # FIXME: this is BAD to not tie specifically to an exception,
-                # but mutagen doesn't provide a base error class to catch
-                pass
+            # Add tags if it doesn't have them.
+            # Most of Mutagen's modules throw an exception if the file already
+            # contains tags, except for mp4. See also
+            # http://code.google.com/p/mutagen/issues/detail?id=101
+            if getattr(raw, 'tags', None) is None:
+                try:
+                    raw.add_tags()
+                except Exception:
+                    # XXX: Not sure needed since we're already checking for
+                    # existence of tags.
+                    pass
 
             # info tags are not actually writable
             for tag in INFO_TAGS:
@@ -238,7 +249,7 @@ class BaseFormat(object):
             return self.mutagen.info.length
         except AttributeError:
             try:
-                return self.mutagen['__length']
+                return self._get_raw()['__length']
             except (KeyError, TypeError):
                 return None
 
@@ -247,7 +258,7 @@ class BaseFormat(object):
             return self.mutagen.info.bitrate
         except AttributeError:
             try:
-                return self.mutagen['__bitrate']
+                return self._get_raw()['__bitrate']
             except (KeyError, TypeError):
                 return None
 
