@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -24,9 +24,10 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-
-
-from xl.metadata._base import BaseFormat
+from xl.metadata._base import (
+    BaseFormat,
+    CoverImage
+)
 from mutagen import flac
 
 class FlacFormat(BaseFormat):
@@ -36,5 +37,12 @@ class FlacFormat(BaseFormat):
     def get_bitrate(self):
         return -1
 
+    def read_tags(self, tags):
+        td = super(FlacFormat, self).read_tags(tags)
+        if 'cover' in tags:
+            td['cover'] = [CoverImage(type=p.type, desc=p.desc, mime=p.mime, data=p.data) \
+                for p in self.mutagen.pictures]
+        return td
+
 # vim: et sts=4 sw=4
 
