@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -34,12 +34,18 @@ class MP4Format(BaseFormat):
     tag_mapping = {
             'title':       '\xa9nam',
             'artist':      '\xa9ART',
+            'albumartist': '\x61ART',
             'album':       '\xa9alb',
+            'composer':    '\xa9wrt',
             'genre':       '\xa9gen',
+            'lyrics':      '\xa9lyr',
+            'encodedby':   '\xa9too',
             'date':        '\xa9day',
             'tracknumber': 'trkn',
             'discnumber':  'disk',
             'copyright':   'cprt',
+            'bpm':         'tmpo',
+            'grouping':    '\xa9grp'
         }
     others = False
     writable = True
@@ -64,6 +70,8 @@ class MP4Format(BaseFormat):
                     f[name].append(tuple(tmp))
             except TypeError:
                 pass
+        elif name == 'tmpo':
+            f[name] = [int(v) for v in value]
         else:
             f[name] = value
 
