@@ -1,5 +1,5 @@
 # -*- coding: utf-8
-# Copyright (C) 2008-2009 Adam Olsen
+# Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -25,16 +25,23 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-import logging
-import time
-import weakref
-import unicodedata
 from copy import deepcopy
 import gio
 import glib
-import gobject
+import logging
+import os.path
+import time
+import unicodedata
+import weakref
+
+from xl import (
+    common,
+    event,
+    metadata,
+    settings
+)
 from xl.nls import gettext as _
-from xl import common, settings, event, metadata
+
 logger = logging.getLogger(__name__)
 
 # map chars to appropriate subsitutes for sorting
@@ -77,7 +84,7 @@ class _MetadataCacher(object):
 
     def __cleanup(self):
         if self._cleanup_id:
-            gobject.source_remove(self._cleanup_id)
+            glib.source_remove(self._cleanup_id)
         current = time.time()
         thresh = current - self.timeout
         for item in self._cache[:]:
@@ -86,7 +93,7 @@ class _MetadataCacher(object):
         if self._cache:
             next_expiry = min([i[2] for i in self._cache])
             timeout = int((next_expiry + self.timeout) - current)
-            self._cleanup_id = gobject.timeout_add_seconds(timeout,
+            self._cleanup_id = glib.timeout_add_seconds(timeout,
                     self.__cleanup)
 
     def add(self, trackobj, formatobj):
@@ -99,7 +106,7 @@ class _MetadataCacher(object):
             least = min([(i[2], i) for i in self._cache])[1]
             self._cache.remove(least)
         if not self._cleanup_id:
-            self._cleanup_id = gobject.timeout_add_seconds(self.timeout,
+            self._cleanup_id = glib.timeout_add_seconds(self.timeout,
                     self.__cleanup)
 
     def remove(self, trackobj):
@@ -136,6 +143,11 @@ class Track(object):
             if there is already a Track for a given uri, we just return
             that Track instance instead of creating a new one.
         """
+        # subclassing interferes with the one-track-per-uri scheme and
+        # with save and restore of tracks, so we disallow it.
+        if cls != Track:
+            raise TypeError, "Track cannot be subclassed!"
+
         uri = None
         if len(args) > 0:
             uri = args[0]
@@ -148,8 +160,8 @@ class Track(object):
         # second TrackDB is consequently ignored. Thus if at all
         # possible, Tracks should NOT be persisted in more than one
         # TrackDB at a time.
+        unpickles = None
         if uri is None:
-            unpickles = None
             if len(args) > 2:
                 unpickles = args[2]
             else:
@@ -162,6 +174,24 @@ class Track(object):
             try:
                 tr = cls.__tracksdict[uri]
                 tr._init = False
+                
+                # if the track *does* happen to be pickled in more than one
+                # place, then we need to preserve any internal tags that aren't
+                # persisted to disk. 
+                #
+                # See https://bugs.launchpad.net/exaile/+bug/1054637
+                if unpickles is None:
+                    if len(args) > 2:
+                        unpickles = args[2]
+                    else:
+                        unpickles = kwargs.get("_unpickles")
+                
+                if unpickles is not None:
+                    for tag, values in unpickles.iteritems():
+                        tags = tr.list_tags()
+                        if tag.startswith('__') and tag not in tags:
+                            tr.set_tag_raw(tag, values)
+                
             except KeyError:
                 tr = object.__new__(cls)
                 cls.__tracksdict[uri] = tr
@@ -229,6 +259,7 @@ class Track(object):
         gloc = gio.File(loc)
         self.__tags['__loc'] = gloc.get_uri()
         self.__register()
+        event.log_event('track_tags_changed', self, '__loc')
 
     def exists(self):
         """
@@ -237,6 +268,15 @@ class Track(object):
         """
         return gio.File(self.get_loc_for_io()).query_exists()
 
+    def get_loc_for_io(self):
+        """
+            Gets the location as a full uri.
+
+            Safe for IO operations via gio, not suitable for display to users
+            as it may be in non-utf-8 encodings.
+        """
+        return self.__tags['__loc']
+
     def local_file_name(self):
         """
             If the file is accessible on the local filesystem, returns a
@@ -246,16 +286,27 @@ class Track(object):
             If a path is returned, it is safe to use for IO operations.
             Existence of a path does *not* guarantee file existence.
         """
-        return gio.File(self.__tags['__loc']).get_path()
+        raise DeprecationWarning('get_local_path() is '
+            'preferred over local_file_name()')
+        return self.get_local_path()
 
-    def get_loc_for_io(self):
+    def get_local_path(self):
         """
-            Gets the location as a full uri.
+            If the file is accessible on a local filesystem, retrieves
+            the full path to it, otherwise nothing.
 
-            Safe for IO operations via gio, not suitable for display to users
-            as it may be in non-utf-8 encodings.
+            :returns: the file path or None
+            :rtype: string or NoneType
         """
-        return self.__tags['__loc']
+        return gio.File(self.get_loc_for_io()).get_path()
+
+    def get_basename(self):
+        """
+            Returns the base name of a resource
+        """
+        gfile = gio.File(self.get_loc_for_io())
+
+        return gfile.get_basename()
 
     def get_type(self):
         """
@@ -276,8 +327,12 @@ class Track(object):
                 return False # not a supported type
             f.write_tags(self.__tags)
             return f
-        except:
-            common.log_exception()
+        except IOError, e:
+            # error writing to the file, probably
+            logger.warning( "Could not write tags to file: %s" % e )
+            return False
+        except Exception:
+            common.log_exception(logger)
             return False
 
     def read_tags(self):
@@ -318,7 +373,7 @@ class Track(object):
             self._dirty = True
             self._scan_valid = True
             return f
-        except:
+        except Exception:
             self._scan_valid = False
             common.log_exception()
             return False
@@ -328,7 +383,8 @@ class Track(object):
             Determines whether a file is accessible on the local filesystem.
         """
         # TODO: determine this better
-        if self.local_file_name():
+        # Maybe use gio.File.is_native()?
+        if self.get_local_path():
             return True
         return False
 
@@ -376,7 +432,7 @@ class Track(object):
         """
             Returns a list of the names of all tags present in this Track.
         """
-        return self.__tags.keys()
+        return self.__tags.keys() + ['__basename']
 
     def set_tag_raw(self, tag, values, notify_changed=True):
         """
@@ -389,18 +445,30 @@ class Track(object):
                 this to False if you know that no other parts of Exaile
                 need to be updated.
         """
-        # handle values that aren't lists
+        if tag == '__loc':
+            logger.warning('Setting "__loc" directly is forbidden, '
+                           'use set_loc() instead.')
+            return
+
+        if tag in ('__basename',):
+            logger.warning('Setting "%s" directly is forbidden.' % tag)
+            return
+
+        # Handle values that aren't lists
         if not isinstance(values, list):
             if not tag.startswith("__"): # internal tags dont have to be lists
                 values = [values]
 
-        # TODO: is this needed? why?
-        # for lists, filter out empty values and convert to unicode
+        # For lists, filter out empty values and convert string values to Unicode
         if isinstance(values, list):
-            values = [common.to_unicode(x, self.__tags.get('__encoding'))
-                for x in values if x not in (None, '')]
+            values = [
+                common.to_unicode(v, self.__tags.get('__encoding')) \
+                    if isinstance(v, basestring) else v \
+                for v in values \
+                    if v not in (None, '')
+            ]
 
-        # save some memory by not storing null values.
+        # Save some memory by not storing null values.
         if not values:
             try:
                 del self.__tags[tag]
@@ -422,10 +490,15 @@ class Track(object):
             :param join: If True, joins lists of values into a
                 single value.
         """
-        val = self.__tags.get(tag)
-        if join and val:
-            return self.join_values(val)
-        return val
+        if tag == '__basename':
+            value = self.get_basename()
+        else:
+            value = self.__tags.get(tag)
+
+        if join and value:
+            return self.join_values(value)
+
+        return value
 
     def get_tag_sort(self, tag, join=True, artist_compilations=True,
             extend_title=True):
@@ -444,48 +517,53 @@ class Track(object):
         # The two magic values here are to ensure that compilations
         # and unknown values are always sorted below all normal
         # values.
-        retval = None
+        value = None
         sorttag = self.__tags.get(tag + "sort")
         if sorttag and tag != "artist":
-            retval = sorttag
+            value = sorttag
         elif tag == "artist":
             if artist_compilations and self.__tags.get('__compilation'):
-                retval = self.__tags.get('albumartist',
+                value = self.__tags.get('albumartist',
                         u"\uffff\uffff\uffff\ufffe")
             else:
-                retval = self.__tags.get('artist',
+                value = self.__tags.get('artist',
                         u"\uffff\uffff\uffff\uffff")
-            if sorttag and retval not in (u"\uffff\uffff\uffff\ufffe",
+            if sorttag and value not in (u"\uffff\uffff\uffff\ufffe",
                     u"\uffff\uffff\uffff\uffff"):
-                retval = sorttag
+                value = sorttag
             else:
                 sorttag = None
         elif tag in ('tracknumber', 'discnumber'):
-            retval = self.split_numerical(self.__tags.get(tag))[0]
+            value = self.split_numerical(self.__tags.get(tag))[0]
         elif tag in ('__length', '__playcount'):
-            retval = self.__tags.get(tag, 0)
+            value = self.__tags.get(tag, 0)
+        elif tag == 'bpm':
+            value = int(self.__tags.get(tag, [0])[0])
+        elif tag == '__basename':
+            # TODO: Check if unicode() is required
+            value = self.get_basename()
         else:
-            retval = self.__tags.get(tag)
+            value = self.__tags.get(tag)
 
-        if not retval:
-            retval = u"\uffff\uffff\uffff\uffff" # unknown
+        if not value:
+            value = u"\uffff\uffff\uffff\uffff" # unknown
             if tag == 'title':
                 gloc = gio.File(self.__tags['__loc'])
                 basename = glib.filename_display_name(gloc.get_basename())
-                retval = u"%s (%s)" % (retval, basename)
+                value = u"%s (%s)" % (value, basename)
         elif not tag.startswith("__") and \
-                tag not in ('tracknumber', 'discnumber'):
+                tag not in ('tracknumber', 'discnumber', 'bpm'):
             if not sorttag:
-                retval = self.format_sort(retval)
+                value = self.format_sort(value)
             else:
-                if isinstance(retval, list):
-                    retval = [self.lower(v + u" " + v) for v in retval]
+                if isinstance(value, list):
+                    value = [self.lower(v + u" " + v) for v in value]
                 else:
-                    retval = self.lower(retval + u" " + v)
+                    value = self.lower(value + u" " + v)
             if join:
-                retval = self.join_values(retval)
+                value = self.join_values(value)
 
-        return retval
+        return value
 
     def get_tag_display(self, tag, join=True, artist_compilations=True,
             extend_title=True):
@@ -505,54 +583,60 @@ class Track(object):
             uri = gio.File(self.__tags['__loc']).get_parse_name()
             return uri.decode('utf-8')
 
-        retval = None
+        value = None
         if tag == "artist":
             if artist_compilations and self.__tags.get('__compilation'):
-                retval = self.__tags.get('albumartist', _VARIOUSARTISTSSTR)
+                value = self.__tags.get('albumartist', _VARIOUSARTISTSSTR)
             else:
-                retval = self.__tags.get('artist', _UNKNOWNSTR)
+                value = self.__tags.get('artist', _UNKNOWNSTR)
         elif tag in ('tracknumber', 'discnumber'):
-            retval = self.split_numerical(self.__tags.get(tag))[0]
+            value = self.split_numerical(self.__tags.get(tag))[0]
         elif tag == '__length':
-            retval = self.__tags.get('__length', 0)
+            value = self.__tags.get('__length', 0)
         elif tag == '__bitrate':
             try:
-                retval = int(self.__tags['__bitrate']) / 1000
-                if retval == -1:
-                    retval = " "
+                value = int(self.__tags['__bitrate']) // 1000
+                if value == -1:
+                    value = " "
                 else:
-                    retval = str(retval) + "k"
-            except:
-                retval = " "
+                    #TRANSLATORS: Bitrate (k here is short for kbps).
+                    value = _("%dk") % value
+            except (KeyError, ValueError):
+                value = " "
+        elif tag == '__basename':
+            value = self.get_basename()
+        elif tag == '__rating':
+            value = unicode(self.get_rating())
         else:
-            retval = self.__tags.get(tag)
+            value = self.__tags.get(tag)
 
-        if not retval:
+        if not value:
             if tag in ['tracknumber', 'discnumber']:
-                return retval
+                return u""
             elif tag in ('__rating', '__playcount'):
-                retval = "0"
+                value = "0"
             else:
-                retval = _UNKNOWNSTR
+                value = _UNKNOWNSTR
                 if tag == 'title':
                     gloc = gio.File(self.__tags['__loc'])
                     basename = glib.filename_display_name(gloc.get_basename())
-                    retval = u"%s (%s)" % (retval, basename)
+                    value = u"%s (%s)" % (value, basename)
 
-        if isinstance(retval, list):
-            retval = [unicode(x) for x in retval]
+        if isinstance(value, list):
+            value = [unicode(x) for x in value]
         else:
-            retval = unicode(retval)
+            value = unicode(value)
 
         if join:
-            retval = self.join_values(retval, _JOINSTR)
+            value = self.join_values(value, _JOINSTR)
 
-        return retval
+        return value
 
     def get_tag_search(self, tag, format=True, artist_compilations=True,
             extend_title=True):
         """
             Get a tag value suitable for passing to the search system.
+            This includes quoting and list joining.
 
             :param format: pre-format into a search query.
             :param artist_compilations: If True, automatically handle
@@ -564,51 +648,54 @@ class Track(object):
         extraformat = ""
         if tag == "artist":
             if artist_compilations and self.__tags.get('__compilation'):
-                retval = self.__tags.get('albumartist', '__null__')
+                value = self.__tags.get('albumartist', '__null__')
                 tag = 'albumartist'
                 extraformat += " ! __compilation==__null__"
             else:
-                retval = self.__tags.get('artist')
+                value = self.__tags.get('artist')
         elif tag in ('tracknumber', 'discnumber'):
-            retval = self.split_numerical(self.__tags.get(tag))[0]
+            value = self.split_numerical(self.__tags.get(tag))[0]
         elif tag in ('__length', '__playcount', '__rating'):
-            retval = self.__tags.get(tag, 0)
+            value = self.__tags.get(tag, 0)
         elif tag == '__bitrate':
             try:
-                retval = int(self.__tags['__bitrate']) / 1000
-                if retval != -1:
-                    retval = str(retval) + "k"
-            except:
-                retval = -1
+                value = int(self.__tags['__bitrate']) // 1000
+                if value != -1:
+                    #TRANSLATORS: Bitrate (k here is short for kbps).
+                    value = _("%dk") % value
+            except (KeyError, ValueError):
+                value = -1
+        elif tag == '__basename':
+            value = self.get_basename()
         else:
-            retval = self.__tags.get(tag)
+            value = self.__tags.get(tag)
 
 
         # Quote arguments
-        if retval is None:
-            retval = '__null__'
+        if value is None:
+            value = '__null__'
             if tag == 'title':
                 extraformat += ' __loc==\"%s\"' % self.__tags['__loc']
-        elif isinstance(retval, list) and format:
-            retval = ['"%s"' % self.quoter(val) for val in retval]
+        elif isinstance(value, list) and format:
+            value = ['"%s"' % self.quoter(val) for val in value]
         elif format:
-            retval = '"%s"' % self.quoter(retval)
+            value = '"%s"' % self.quoter(value)
 
         # Join lists
         if format:
-            if isinstance(retval, list):
-                retval = " ".join(['%s==%s'%(tag, val) for val in retval])
+            if isinstance(value, list):
+                value = " ".join(['%s==%s' % (tag, v) for v in value])
             else:
-                retval = '%s==%s' % (tag, retval)
+                value = '%s==%s' % (tag, value)
             if extraformat:
-                retval += extraformat
+                value += extraformat
 
         # hack to make things work - discnumber breaks without it.
         # TODO: figure out why this happens, cleaner solution
-        if not isinstance(retval, list) and not tag.startswith("__"):
-            retval = unicode(retval)
+        if not isinstance(value, list) and not tag.startswith("__"):
+            value = unicode(value)
 
-        return retval
+        return value
 
     def get_tag_disk(self, tag):
         """
@@ -621,7 +708,7 @@ class Track(object):
         if not f:
             try:
                 f = metadata.get_format(self.get_loc_for_io())
-            except:
+            except Exception: # TODO: What exception?
                 return None
             if not f:
                 return None
@@ -640,7 +727,7 @@ class Track(object):
         if not f:
             try:
                 f = metadata.get_format(self.get_loc_for_io())
-            except:
+            except Exception: # TODO: What exception?
                 return None
             if not f:
                 return None
@@ -654,17 +741,17 @@ class Track(object):
     def get_rating(self):
         """
             Returns the current track rating as an integer, as
-            determined by the ``miscellaneous/rating_steps`` setting.
+            determined by the ``rating/maximum`` setting.
         """
         try:
             rating = float(self.get_tag_raw('__rating'))
         except (TypeError, KeyError, ValueError):
             return 0
 
-        steps = settings.get_option("miscellaneous/rating_steps", 5)
-        rating = int(round(rating*float(steps)/100.0))
+        maximum = settings.get_option("rating/maximum", 5)
+        rating = int(round(rating*float(maximum)/100.0))
 
-        if rating > steps: return int(steps)
+        if rating > maximum: return int(maximum)
         elif rating < 0: return 0
 
         return rating
@@ -672,14 +759,17 @@ class Track(object):
     def set_rating(self, rating):
         """
             Sets the current track rating from an integer, on the
-            scale determined by the ``miscellaneous/rating_steps`` setting.
+            scale determined by the ``rating/maximum`` setting.
+            
+            Returns the scaled rating
         """
         rating = float(rating)
-        steps = float(settings.get_option("miscellaneous/rating_steps", 5))
-        rating = min(rating, steps)
+        maximum = float(settings.get_option("rating/maximum", 5))
+        rating = min(rating, maximum)
         rating = max(0, rating)
-        rating = float(rating * 100.0 / steps)
+        rating = float(rating * 100.0 / maximum)
         self.set_tag_raw('__rating', rating)
+        return rating
 
     ### Special functions for wrangling tag values ###
 
@@ -702,7 +792,7 @@ class Track(object):
         """
         if type(values) in (str, unicode):
             return values
-        return glue.join(values)
+        return glue.join(map(common.to_unicode, values))
 
     @staticmethod
     def split_numerical(values):
@@ -737,7 +827,8 @@ class Track(object):
             stripping the chars leaves nothing the original field is
             returned with only whitespace removed.
         """
-        stripped = value.lstrip(" `~!@#$%^&*()_+-={}|[]\\\";'<>?,./")
+        stripped = common.to_unicode(value).lstrip(
+            " `~!@#$%^&*()_+-={}|[]\\\";'<>?,./")
         if stripped:
             return stripped
         else:
@@ -817,7 +908,7 @@ class Track(object):
             update the cached the_cutter values
         """
         if data == "collection/strip_list":
-            cls.__the_cuts = settings.get_option('collection/strip_list', [])
+            cls._Track__the_cuts = settings.get_option('collection/strip_list', [])
 
 
 
