@@ -1,4 +1,4 @@
-# Copyright (C) 2009 Aren Olson
+# Copyright (C) 2009-2010 Aren Olson
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -25,66 +25,71 @@
 # from your version.
 
 GNOME_MMKEYS = None
-EXAILE = None
 
-from xl import common, event
+from xl import common, event, player
 import dbus, logging, traceback
 logger = logging.getLogger(__name__)
 
 def callback(key):
-    global EXAILE
     if key in ('Play', 'PlayPause', 'Pause'):
-        if EXAILE.player.is_playing() or EXAILE.player.is_paused():
-            EXAILE.player.toggle_pause()
+        if player.PLAYER.is_playing() or player.PLAYER.is_paused():
+            player.PLAYER.toggle_pause()
         elif key != "Pause":
-            EXAILE.queue.play()
+            player.QUEUE.play()
         else:
             pass
     elif key == 'Stop':
-        EXAILE.player.stop()
+        player.PLAYER.stop()
     elif key == 'Previous':
-        EXAILE.queue.prev()
+        player.QUEUE.prev()
     elif key == 'Next':
-        EXAILE.queue.next()
+        player.QUEUE.next()
 
 def enable(exaile):
     if exaile.loading:
         event.add_callback(_enable, "player_loaded")
     else:
-        _enable(None, exaile, None)
+        _enable(None, player.PLAYER, None)
 
-def _enable(eventname, exaile, nothing):
-    global GNOME_MMKEYS, EXAILE
-    EXAILE = exaile
+def _enable(eventname, player, nothing):
+    global GNOME_MMKEYS
     def on_gnome_mmkey(app, key):
         if app == "Exaile":
             callback(key)
     try:
         bus = dbus.SessionBus()
         try:
-            # new method (for gnome 2.22.x)
-            obj = bus.get_object('org.gnome.SettingsDaemon',
-                '/org/gnome/SettingsDaemon/MediaKeys')
-            GNOME_MMKEYS = gnome = dbus.Interface(obj,
-                'org.gnome.SettingsDaemon.MediaKeys')
-            gnome.GrabMediaPlayerKeys("Exaile", 0)
-            gnome.connect_to_signal('MediaPlayerKeyPressed', on_gnome_mmkey)
-            return True
-        except:
-            traceback.print_exc()
-            # old method
-            obj = bus.get_object('org.gnome.SettingsDaemon',
-                '/org/gnome/SettingsDaemon')
-            GNOME_MMKEYS = gnome = dbus.Interface(obj,
-                'org.gnome.SettingsDaemon')
-            gnome.GrabMediaPlayerKeys("Exaile", 0)
-            gnome.connect_to_signal('MediaPlayerKeyPressed', on_gnome_mmkey)
-            return True
-    except:
-        disable(exaile) #disconnect if we failed to load completely
+            # Mate desktop
+            obj = bus.get_object('org.mate.SettingsDaemon',
+                '/org/mate/SettingsDaemon/MediaKeys')
+            GNOME_MMKEYS = dbus.Interface(obj,
+                'org.mate.SettingsDaemon.MediaKeys')
+        except dbus.DBusException:
+            try:
+                # New method (for Gnome 2.22.x)
+                obj = bus.get_object('org.gnome.SettingsDaemon',
+                    '/org/gnome/SettingsDaemon/MediaKeys')
+                GNOME_MMKEYS = dbus.Interface(obj,
+                    'org.gnome.SettingsDaemon.MediaKeys')
+            except dbus.DBusException:
+                try:
+                    # Old method
+                    obj = bus.get_object('org.gnome.SettingsDaemon',
+                        '/org/gnome/SettingsDaemon')
+                    GNOME_MMKEYS = dbus.Interface(obj,
+                        'org.gnome.SettingsDaemon')
+                except dbus.DBusException:
+                    # Make sure we bail out completely here
+                    raise
+    except dbus.DBusException:
+        disable(None) # Disconnect if we failed to load completely
         GNOME_MMKEYS = None
         common.log_exception(logger)
         return False
+    else:
+        GNOME_MMKEYS.GrabMediaPlayerKeys("Exaile", 0)
+        GNOME_MMKEYS.connect_to_signal('MediaPlayerKeyPressed', on_gnome_mmkey)
+        return True
 
 def disable(exaile):
     global GNOME_MMKEYS
