@@ -10,7 +10,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 # Arunas Radzvilavicius, arunas.rv@gmail.com
 #    TODO: download books?
@@ -22,9 +22,20 @@
 import gtk, gobject, os
 import librivoxsearch as LS
 import about_window as AW
-from xl import trax, playlist, event, xdg, common, settings
-from xlgui import guiutil
-
+from xl import (
+    common,
+    event,
+    playlist,
+    settings,
+    trax,
+    xdg
+)
+from xlgui import (
+    guiutil,
+    icons,
+    main
+)
+from xlgui.widgets.common import DragTreeView
 
 def enable(exaile):
     if exaile.loading:
@@ -67,9 +78,9 @@ class LVPanel():
         return
 
     @common.threaded
-    def get_chapters_and_add(self, row, location=None):
+    def get_chapters_and_add(self, row):
         self.get_all(row)
-        self.add_to_playlist(self.books[row].chapters, location)
+        self.add_to_playlist(self.books[row].chapters)
         return
 
     @guiutil.idle_add()
@@ -87,10 +98,10 @@ class LVPanel():
         self.librivoxdir= os.path.dirname(__file__)
         self.abicon = gtk.gdk.pixbuf_new_from_file(self.librivoxdir+'/ebook.png')
         self.clock_icon=gtk.gdk.pixbuf_new_from_file(self.librivoxdir+'/clock.png')
-        self.chapter_icon=gtk.gdk.pixbuf_new_from_file(xdg.get_data_path('images/track.png'))
+        self.chapter_icon=icons.MANAGER.pixbuf_from_icon_name(
+            'audio-x-generic', gtk.ICON_SIZE_SMALL_TOOLBAR)
         self.gui_init(exaile)
         self.connect_events()
-        self.controller=exaile.gui.main
         self.getting_info=False
 
     def gui_init(self, exaile):
@@ -181,18 +192,21 @@ class LVPanel():
             return
         if len(path)==1: # selected item is book
             if self.books[row].loaded: # book info already loaded
-                self.add_to_playlist(self.books[row].chapters, location=None)
+                self.add_to_playlist(self.books[row].chapters)
             else:
-                self.get_chapters_and_add(row, location=None)
+                self.get_chapters_and_add(row)
 
         if len(path)>1: # selected item is chapter
             chapter=self.books[path[0]].chapters[path[1]]
-            self.add_to_playlist([chapter], location=None)
+            self.add_to_playlist([chapter])
 
-    def add_to_playlist(self, chapters, location=None):
-        current_playlist=self.controller.get_selected_playlist().playlist
-        tracks=self.generate_tracks(chapters)
-        current_playlist.add_tracks(tracks, location)
+    def add_to_playlist(self, chapters):
+        current_playlist = main.get_selected_playlist()
+        if not current_playlist:
+            return
+        
+        tracks = self.generate_tracks(chapters)
+        current_playlist.playlist.extend(tracks)
 
 
     def menu_popup(self, treeview, event):
@@ -225,7 +239,7 @@ class LVPanel():
             if book.loaded: #chapters loaded
                 for chapter in book.chapters:
                     chapter_track=self.generate_tracks([chapter])
-                    guiutil.DragTreeView.dragged_data[chapter[1]] = chapter_track[0]
+                    DragTreeView.dragged_data[chapter[1]] = chapter_track[0]
                     uris.append(chapter[1])
                 selection.set_uris(uris)
                 del uris
@@ -233,8 +247,12 @@ class LVPanel():
             else:
                 if book.is_loading:
                     return
-                current_playlist=self.controller.get_selected_playlist()
-                current_playlist_tv=self.controller.get_selected_playlist().list
+                    
+                current_playlist = main.get_selected_playlist()
+                if not current_playlist:
+                    return
+                
+                current_playlist_tv = current_playlist.list
 
                 (x,y)=current_playlist_tv.get_pointer()
                 rect=current_playlist_tv.get_allocation()
@@ -260,7 +278,7 @@ class LVPanel():
                 return
             chapter=self.books[path[0]].chapters[path[1]]
             chapter_track=self.generate_tracks([chapter])
-            guiutil.DragTreeView.dragged_data[chapter[1]] = chapter_track[0]
+            DragTreeView.dragged_data[chapter[1]] = chapter_track[0]
             uri=chapter[1]
             selection.set('text/uri-list', 0, uri)
 
@@ -343,6 +361,6 @@ class Status():
         msg_id=self.bar.push(context_id, status)
         return (context_id, msg_id)
     def unset_status(self, context_id, msg_id):
-        self.bar.remove(context_id, msg_id)
+        self.bar.remove_message(context_id, msg_id)
 
 
